﻿using System.Collections;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Scheduler;

namespace LinqToSqlProvider.ObjectModel.Provider
{
    public abstract class SchedulerProviderBase : DependencyObject, ISchedulerProvider
    {
        private RadScheduler scheduler;

        #region ISchedulerProvider Members

        public void Attach(RadScheduler schedulerToAttach)
        {
            scheduler = schedulerToAttach;
            UpdateAppointments();
            HookToSchedulerEvents();
        }

        /// <summary>
        /// Detaches the current <see cref="RadScheduler" /> from this instance.
        /// </summary>
        public void Detach()
        {
            if (scheduler != null)
            {
                UnHookToSchedulerEvents();
                scheduler.AppointmentsSource = null;
            }
        }

        #endregion

        /// <summary>
        /// Gets the appointments from the current data source.
        /// </summary>
        /// <returns>A collection with the appointments.</returns>
        public abstract IEnumerable GetAppointments();

        /// <summary>
        /// Updates the appointments source of the scheduler.
        /// </summary>
        public void UpdateAppointments()
        {
            if (scheduler != null)
                scheduler.AppointmentsSource = GetAppointments();
        }

        protected virtual void OnAppointmentAdded(IAppointment appointment)
        {
        }

        protected virtual void OnAppointmentAdding(IAppointment appointment)
        {
        }

        protected virtual void OnAppointmentCreated(IAppointment appointment)
        {
        }

        protected virtual IAppointment OnAppointmentCreating()
        {
            return new SqlAppointment();
        }

        protected virtual void OnAppointmentDeleted(IAppointment appointment)
        {
        }

        protected virtual void OnAppointmentDeleting(IAppointment appointment)
        {
        }

        protected virtual void OnAppointmentEdited(IAppointment appointment)
        {
        }

        protected virtual void OnAppointmentEditing(IAppointment appointment)
        {
        }

        private void OnSchedulerAppointmentAdded(object sender, AppointmentAddedEventArgs e)
        {
            OnAppointmentAdded(e.Appointment);
        }

        private void OnSchedulerAppointmentAdding(object sender, AppointmentAddingEventArgs e)
        {
            OnAppointmentAdding(e.Appointment);
        }

        private void OnSchedulerAppointmentCreated(object sender, AppointmentCreatedEventArgs e)
        {
            OnAppointmentCreated(e.CreatedAppointment);
        }

        private void OnSchedulerAppointmentCreating(object sender, AppointmentCreatingEventArgs e)
        {
            e.NewAppointment = OnAppointmentCreating();
        }

        private void OnSchedulerAppointmentDeleted(object sender, AppointmentDeletedEventArgs e)
        {
            OnAppointmentDeleted(e.Appointment);
        }

        private void OnSchedulerAppointmentDeleting(object sender, AppointmentDeletingEventArgs e)
        {
            OnAppointmentDeleting(e.Appointment);
        }

        private void OnSchedulerAppointmentEdited(object sender, AppointmentEditedEventArgs e)
        {
            OnAppointmentEdited(e.Appointment);
        }

        private void OnSchedulerAppointmentEditing(object sender, AppointmentEditingEventArgs e)
        {
            OnAppointmentEditing(e.Appointment);
        }

        private void HookToSchedulerEvents()
        {
            scheduler.AppointmentAdded += OnSchedulerAppointmentAdded;
            scheduler.AppointmentAdding += OnSchedulerAppointmentAdding;
            scheduler.AppointmentCreated += OnSchedulerAppointmentCreated;
            scheduler.AppointmentCreating += OnSchedulerAppointmentCreating;
            scheduler.AppointmentDeleted += OnSchedulerAppointmentDeleted;
            scheduler.AppointmentDeleting += OnSchedulerAppointmentDeleting;
            scheduler.AppointmentEdited += OnSchedulerAppointmentEdited;
            scheduler.AppointmentEditing += OnSchedulerAppointmentEditing;
        }

        private void UnHookToSchedulerEvents()
        {
            scheduler.AppointmentAdded -= OnSchedulerAppointmentAdded;
            scheduler.AppointmentAdding -= OnSchedulerAppointmentAdding;
            scheduler.AppointmentCreated -= OnSchedulerAppointmentCreated;
            scheduler.AppointmentCreating -= OnSchedulerAppointmentCreating;
            scheduler.AppointmentDeleted -= OnSchedulerAppointmentDeleted;
            scheduler.AppointmentDeleting -= OnSchedulerAppointmentDeleting;
            scheduler.AppointmentEdited -= OnSchedulerAppointmentEdited;
            scheduler.AppointmentEditing -= OnSchedulerAppointmentEditing;
        }
    }
}