﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Telerik.Windows.Controls.Scheduler;
using Telerik.Windows.Controls.Scheduler.ICalendar;

namespace LinqToSqlProvider.ObjectModel.Provider
{
    internal class SqlSchedulerProvider : SchedulerProviderBase
    {
        private readonly ScheduleDataContext dataContext;

        /// <summary>
        /// Initializes a new instance of the <see cref="SqlSchedulerProvider"/> class.
        /// </summary>
        public SqlSchedulerProvider()
        {
            this.dataContext = new ScheduleDataContext();
        }

        /// <summary>
        /// Gets the appointments from the current data source.
        /// </summary>
        /// <returns>A collection with the appointments.</returns>
        public override IEnumerable GetAppointments()
        {
            IEnumerable<SqlAppointment> appointments =
                this.dataContext.SqlAppointments.Where(a => a.Type == (int)AppointmentType.Regular);
            this.ApplyRecurrenceRulesExceptions(appointments);
            this.HookAppointmentEvents(appointments);
            var appointmentsCollection = new ObservableCollection<SqlAppointment>(appointments);
            return appointmentsCollection;
        }

        /// <summary>
        /// Called when <see cref="Appointment"/> is added.
        /// </summary>
        /// <param name="appointment">The appointment.</param>
        protected override void OnAppointmentAdded(IAppointment appointment)
        {
            base.OnAppointmentAdded(appointment);

            this.AddAppointmentToDataBase(appointment);
        }

        internal void AddAppointmentToDataBase(IAppointment appointment)
        {
            var appointmentToAdd = (SqlAppointment)appointment;

            if (appointmentToAdd.RecurrenceRule != null)
                appointmentToAdd.RecurrencePattern = CalHelper.RecurrencePatternToString(appointmentToAdd.RecurrenceRule.Pattern);

            this.dataContext.SqlAppointments.InsertOnSubmit(appointmentToAdd);
            this.dataContext.SubmitChanges();
        }

        /// <summary>
        /// Called when <see cref="Appointment"/> is deleted.
        /// </summary>
        /// <param name="appointment">The appointment.</param>
        protected override void OnAppointmentDeleted(IAppointment appointment)
        {
            base.OnAppointmentDeleted(appointment);

            // unhook from RecurrenceRuleChanged event
            appointment.RecurrenceRuleChanged -= this.OnRecurrenceRuleChanged;

            // delete exceptions
            DeleteAppointmentExceptionsFromDataContext(appointment);

            this.dataContext.SqlAppointments.DeleteOnSubmit((SqlAppointment)appointment);
            this.dataContext.SubmitChanges();
        }

        /// <summary>
        /// Called when <see cref="Appointment"/> is edited.
        /// </summary>
        /// <param name="appointment">The appointment.</param>
        protected override void OnAppointmentEdited(IAppointment appointment)
        {
            base.OnAppointmentEdited(appointment);

            this.dataContext.SubmitChanges();
        }

        private void ApplyRecurrenceRulesExceptions(IEnumerable<SqlAppointment> appointments)
        {
            IQueryable<AppointmentException> appointmentExceptions = this.dataContext.AppointmentExceptions;
            foreach (AppointmentException appointmentException in appointmentExceptions)
            {
                SqlAppointment currentSqlAppointment = appointments.Single(a => a.UniqueId == appointmentException.MasterUniqueId);

                if (appointmentException.AppointmentUniqueId == null)
                {
                    currentSqlAppointment.RecurrenceRule.AddException(appointmentException.Date);
                }
                else
                {
                    currentSqlAppointment.RecurrenceRule.AddException(appointmentException.Date, appointmentException.SqlAppointment);
                }
            }
        }

        private static void DeleteAppointmentExceptionsFromDataContext(IAppointment appointment)
        {
            var appointmentToModify = appointment as SqlAppointment;
            if (appointmentToModify != null)
            {
                if (appointmentToModify.RecurrenceRule != null)
                {
                    // delete all exceptions from Exceptions table
                    // cascade delete rule deletes all related records in SqlAppointments table
                    appointmentToModify.RecurrenceRuleExceptions.Clear();
                }
            }
        }

        private void HookAppointmentEvents(IEnumerable<SqlAppointment> appointments)
        {
            foreach (SqlAppointment appointment in appointments)
            {
                appointment.RecurrenceRuleChanged += this.OnRecurrenceRuleChanged;
            }
        }

        private void OnRecurrenceRuleChanged(object sender, EventArgs args)
        {
            var changedAppointment = sender as SqlAppointment;
            if (changedAppointment != null)
            {
                DeleteAppointmentExceptionsFromDataContext(changedAppointment);
                if (changedAppointment.RecurrenceRule != null)
                {
                    // add date exceptions to the Exceptions table in the database
                    foreach (ExceptionOccurrence exception in changedAppointment.RecurrenceRule.Exceptions.Where(e => e.Appointment == null))
                    {
                        changedAppointment.RecurrenceRuleExceptions.Add(
                            new AppointmentException
                                {
                                    MasterUniqueId = changedAppointment.UniqueId,
                                    AppointmentUniqueId = null,
                                    Date = exception.ExceptionDate
                                });
                    }

                    // add exceptions with appointments to the database
                    foreach (ExceptionOccurrence exception in changedAppointment.RecurrenceRule.Exceptions.Where(e => e.Appointment != null))
                    {
                        var exceptionAppointment = exception.Appointment.Copy() as SqlAppointment;
                        if (exceptionAppointment != null)
                        {
                            exceptionAppointment.UniqueId = Guid.NewGuid();
                            exceptionAppointment.RecurrenceRule = null;
                            exceptionAppointment.Type = (int)AppointmentType.Exception;
                            exceptionAppointment.RecurrencePattern = null;

                            changedAppointment.RecurrenceRuleExceptions.Add(
                                new AppointmentException
                                    {
                                        SqlAppointment = exceptionAppointment,
                                        MasterUniqueId = changedAppointment.UniqueId,
                                        AppointmentUniqueId = exceptionAppointment.UniqueId,
                                        Date = exception.ExceptionDate
                                    });
                        }
                    }

                    changedAppointment.RecurrencePattern = CalHelper.RecurrencePatternToString(changedAppointment.RecurrenceRule.Pattern);
                    this.dataContext.SubmitChanges();
                }
            }
        }
    }
}